package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import static de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType.INTERSECTION;
import static de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType.IS_NOT_IN_CONTACT;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import java.util.Optional;

/**
 * The intersection creator implementation of {@link BoundaryBiCreaterType#INTERSECTION} for {@link
 * AbstractGrowingBoundary}. <br> The result is a new boundary with intersection of both boundaries,
 * or an empty optional, if the creation is not possible. <br> In: {@code b1: [10..20]; b2:
 * [15..30]} <br> Out: {@code [15..20]} <br>
 */
public class BiCreaterIntersection extends AbstractBoundaryBiCreater<AbstractGrowingBoundary> {

  private static final BiCreaterIntersection instance = new BiCreaterIntersection();

  private BiCreaterIntersection() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static BiCreaterIntersection getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return INTERSECTION;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(
      AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    if (b1.checkWith(IS_NOT_IN_CONTACT, b2)) {
      return Optional.empty();
    }
    int compLower = b1.compareToLowerBound(b2);
    int compUpper = b1.compareToUpperBound(b2);
    return Optional.of(
        b1.getCopy(
            compLower > 0 ? b1.getLowerBound() : b2.getLowerBound(),
            compUpper < 0 ? b1.getUpperBound() : b2.getUpperBound(),
            compLower > 0 ? b1.getLowerBoundType() : b2.getLowerBoundType(),
            compUpper < 0 ? b1.getUpperBoundType() : b2.getUpperBoundType()));
  }
}
