package de.unikoblenz.fgbks.base.utils.boundary.bicreater;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

/**
 * The between creator implementation of {@link BoundaryBiCreaterType#BETWEEN} for {@link
 * StringBoundary}. <br> The result is a new boundary with boundary between the both boundaries, or
 * an empty optional, if the creation is not possible. <br> In: {@code b1: "a"; b2: "b"} <br> Out:
 * {@code not("a","b")} <br>
 */
public class BiCreaterStringBetween extends AbstractBoundaryBiCreater<StringBoundary> {

  private static final BiCreaterStringBetween instance = new BiCreaterStringBetween();

  private BiCreaterStringBetween() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static BiCreaterStringBetween getInstance() {
    return instance;
  }

  @Override
  public BoundaryBiCreaterType getType() {
    return BoundaryBiCreaterType.BETWEEN;
  }

  @Override
  public Optional<StringBoundary> create(StringBoundary b1, StringBoundary b2) {
    if (b1.matchesAny() || b2.matchesAny()) {
      return Optional.empty();
    }
    if (b1.matchesNoneOfValues() && b2.matchesNoneOfValues()) {
      Set<String> vals1 = new HashSet<>(Arrays.asList(b1.getValues()));
      Set<String> vals2 = new HashSet<>(Arrays.asList(b2.getValues()));
      Set<String> values = new HashSet<>();
      for (String v1 : vals1) {
        if (vals2.contains(v1)) {
          values.add(v1);
        }
      }
      if (values.size() == 0) {
        return Optional.empty();
      }
      return Optional.of(StringBoundary.getBuilder().addValues(values).build());
    }
    if (b1.matchesNoneOfValues()) {
      return getOneIsNoneOfValues(b1, b2);
    }
    if (b2.matchesNoneOfValues()) {
      return getOneIsNoneOfValues(b2, b1);
    }
    return Optional.of(
        StringBoundary.getBuilder()
            .matchesNoneOfValues(true)
            .addValues(b1.getValues())
            .addValues(b2.getValues())
            .build());
  }

  private Optional<StringBoundary> getOneIsNoneOfValues(StringBoundary b1, StringBoundary b2) {
    Set<String> values = new HashSet<>(Arrays.asList(b1.getValues()));
    values.removeAll(Arrays.asList(b2.getValues()));
    if (values.size() == 0) {
      return Optional.empty();
    }
    return Optional.of(StringBoundary.getBuilder().addValues(values).build());
  }
}
