package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;

/**
 * The not in contact checker implementation of {@link BoundaryCheckType#IS_NOT_IN_CONTACT} for
 * {@link AbstractGrowingBoundary}. <br> The check returns true, if the first boundary is not in
 * contact with the other boundary. <br> E.g: <br>
 *
 * <ul>
 *   <li>b1 = "= 1"; b2 = "= 1" -{@literal >} false
 *   <li>b1 = "{@literal <}= 1"; b2 = "{@literal <} 2" -{@literal >} false
 *   <li>b1 = "[0..20]"; b2 = "]20..22]" -{@literal >} true
 * </ul>
 */
public class CheckNotInContact extends AbstractBoundaryCheck<AbstractGrowingBoundary> {

  private static final CheckNotInContact instance = new CheckNotInContact();

  private CheckNotInContact() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CheckNotInContact getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_NOT_IN_CONTACT;
  }

  @Override
  public boolean check(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    return !CheckInContact.checkInContact(b1, b2);
  }

  /**
   * see {@link CheckNotInContact}
   */
  public static boolean checkNotInContact(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    return !CheckInContact.checkInContact(b1, b2);
  }
}
