package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;

/**
 * The overlap checker implementation of {@link BoundaryCheckType#IS_OVERLAPPING} for {@link
 * AbstractGrowingBoundary}. <br> The check returns true, if the first boundary is overlapping the
 * other boundary (not equal and no subsumption). <br> E.g: <br>
 *
 * <ul>
 *   <li>b1 = "{@literal <}= 1"; b2 = "{@literal >}= 0" -{@literal >} true
 *   <li>b1 = "{@literal <}= 1"; b2 = "{@literal <} 2" -{@literal >} false (for integer/long) / true for double
 *   <li>b1 = "[0..20]"; b2 = "[4..22]" -{@literal >} true
 * </ul>
 */
public class CheckOverlap extends AbstractBoundaryCheck<AbstractGrowingBoundary> {

  private static final CheckOverlap instance = new CheckOverlap();

  private CheckOverlap() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CheckOverlap getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_OVERLAPPING;
  }

  @Override
  public boolean check(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    return checkOverlapping(b1, b2);
  }

  /** see {@link CheckOverlap} */
  public static boolean checkOverlapping(AbstractGrowingBoundary b1, AbstractGrowingBoundary b2) {
    return CheckInContact.checkInContact(b1, b2)
        && !CheckEqual.checkEqual(b1, b2)
        && !CheckIsSubsumption.checkIsSubsumption(b1, b2);
  }
}
