package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;

/**
 * The overlap checker implementation of {@link BoundaryCheckType#IS_OVERLAPPING} for {@link
 * StringBoundary}. <br> The check returns true, if the first boundary is overlapping with the other
 * boundary (not equal and no subsumption). <br> E.g: <br>
 *
 * <ul>
 *   <li>b1 = "a"; b2 = "a" -{@literal >} false
 *   <li>b1 = "not("a","b"); b2 = "not("a","b")" -{@literal >} false
 *   <li>b1 = "a","b"; b2 = "b" -{@literal >} true
 * </ul>
 */
public class CheckStringOverlap extends AbstractBoundaryCheck<StringBoundary> {

  private static CheckStringOverlap instance = new CheckStringOverlap();

  private CheckStringOverlap() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CheckStringOverlap getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_NOT_IN_CONTACT;
  }

  @Override
  public boolean check(StringBoundary b1, StringBoundary b2) {
    return checkSubsumption(b1, b2);
  }

  /** see {@link CheckStringOverlap} */
  public static boolean checkSubsumption(StringBoundary b1, StringBoundary b2) {
    return CheckStringInContact.checkInContact(b1, b2)
        && !CheckStringEqual.checkEqual(b1, b2)
        && !CheckStringSubsumption.checkSubsumption(b1, b2);
  }
}
