package de.unikoblenz.fgbks.base.utils.boundary.creater;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import java.util.Optional;

/**
 * The upper creator implementation of {@link BoundaryCreaterType#LOWER} for {@link
 * StringBoundary}.
 * <br>
 * The result is the set of all strings which are NOT the current StringBoundary.
 */
public class CreaterStringLower extends AbstractBoundaryCreater<StringBoundary> {

  private static final CreaterStringLower instance = new CreaterStringLower();

  private CreaterStringLower() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CreaterStringLower getInstance() {
    return instance;
  }

  @Override
  public BoundaryCreaterType getType() {
    return BoundaryCreaterType.LOWER;
  }

  @Override
  public Optional<StringBoundary> create(StringBoundary b) {
    return Optional.of(
        StringBoundary.getBuilder()
            .addValues(b.getValues())
            .matchesNoneOfValues(!b.matchesNoneOfValues())
            .matchesAny(b.getAmountOfElements() == 0)
            .build());
  }
}
