package de.unikoblenz.fgbks.base.utils.boundary.creater;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.BoundType;
import java.util.Optional;

/**
 * The upper creator implementation of {@link BoundaryCreaterType#UPPER} for {@link
 * AbstractGrowingBoundary}. <br> The result is the complete range of values, which are higher than
 * the current boundary.<br> In: {@code <=5} <br> Out: {@code >5} <br>
 */
public class CreaterUpper extends AbstractBoundaryCreater<AbstractGrowingBoundary> {

  private static final CreaterUpper instance = new CreaterUpper();

  private CreaterUpper() {
    super();
  }

  /**
   * Get the current instance of this class.
   *
   * @return the singleton instance
   */
  public static CreaterUpper getInstance() {
    return instance;
  }

  @Override
  public BoundaryCreaterType getType() {
    return BoundaryCreaterType.UPPER;
  }

  @Override
  public Optional<AbstractGrowingBoundary> create(AbstractGrowingBoundary b) {
    if (b.getUpperBound().equals(b.getMaxValue())) {
      return Optional.empty();
    }
    return Optional.of(
        b.getCopy(
            b.getUpperBound(),
            b.getMaxValue(),
            b.getUpperBoundType().getOp(),
            BoundType.INCLUSIVE));
  }
}
