package de.unikoblenz.fgbks.base.utils.boundary.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.BoundType;
import de.unikoblenz.fgbks.base.utils.boundary.BoundaryParseException;

/**
 * A Boolean boundary can be only true, false or both (true and false = empty String).
 */
public class BooleanBoundary extends AbstractGrowingBoundary<Boolean> {

  /**
   * Parse the input text to a new Boolean boundary. See {@link BooleanBoundary} for the required
   * structure.
   *
   * @param text the text, which will be parsed to the new BooleanBoundary object
   */
  public BooleanBoundary(String text) {
    super(text, Boolean.TYPE);
  }

  private BooleanBoundary() {
    super(Boolean.TYPE);
  }

  @Override
  public Boolean getMinValue() {
    return false;
  }

  @Override
  public Boolean getMaxValue() {
    return true;
  }

  @Override
  public AbstractGrowingBoundary<Boolean> getCopy(
      Boolean lowerBound, Boolean upperBound, BoundType lowerBoundType, BoundType upperBoundType) {
    BooleanBoundary v = new BooleanBoundary();
    v.lowerBound = lowerBound;
    v.upperBound = upperBound;
    v.lowerBoundType = lowerBoundType;
    v.upperBoundType = upperBoundType;
    v.validate();
    return v;
  }

  @Override
  protected void parse(String text) {
    upperBoundType = INCLUSIVE;
    lowerBoundType = INCLUSIVE;
    if (text.trim().isEmpty()) {
      lowerBound = getMinValue();
      upperBound = getMaxValue();
    } else if (Boolean.TRUE.toString().equals(text.toLowerCase())) {
      lowerBound = upperBound = getMaxValue();
    } else if (Boolean.FALSE.toString().equals(text.toLowerCase())) {
      lowerBound = upperBound = getMinValue();
    } else {
      throw new BoundaryParseException("Boolean value cloud not be parsed: " + text);
    }
  }

  /**
   * Get the boolean boundary: "true", "false" or "".
   *
   * @return the string representation of the boolean Boundary
   */
  @Override
  public String getParsedText() {
    return lowerBound != upperBound ? "" : lowerBound.toString();
  }
}
