package de.unikoblenz.fgbks.base.utils.boundary.impl;

import de.unikoblenz.fgbks.base.utils.boundary.Boundary;
import de.unikoblenz.fgbks.base.utils.boundary.bicreater.BoundaryBiCreaterType;
import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType;
import de.unikoblenz.fgbks.base.utils.boundary.creater.BoundaryCreaterType;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import java.util.Optional;

/**
 * A invalid boundary, which is used, if e.g. no boundary can be parsed.
 */
public class InvalidBoundary implements Boundary {

  private static InvalidBoundary instance = new InvalidBoundary();

  private InvalidBoundary() {
  }

  /**
   * Get the singleton instance of the invalid boundary.
   *
   * @return the invalid boundary instance
   */
  public static InvalidBoundary getInstance() {
    return instance;
  }

  @Override
  public VTypeRef getTypeRef() {
    return null;
  }

  @Override
  public String getText() {
    return "null";
  }

  @Override
  public String getParsedText() {
    return "";
  }

  @Override
  public boolean checkWith(BoundaryCheckType checkType, Boundary other) {
    return false;
  }

  @Override
  public Optional<Boundary> create(BoundaryCreaterType createType) {
    return Optional.empty();
  }

  @Override
  public Optional<Boundary> createBi(BoundaryBiCreaterType biCreaterType, Boundary other) {
    return Optional.empty();
  }

  @Override
  public int compareTo(Boundary o) {
    return 0;
  }
}
